@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclExtdInt.h,v 1.3 2002/09/26 00:19:18 hobbs Exp $
+ * $Id: tclExtdInt.h,v 1.6 2005/04/26 20:01:33 hobbs Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -21,6 +21,10 @@
 
 #include "tclExtend.h"
 
+/* Keep it before tcl*Port, otherwise a clash in TclpPanic.
+ */
+#include "tclInt.h"
+
 #if defined(__WIN32__) || defined(_WIN32)
 #   include "tclXwinPort.h"
 #else
@@ -28,10 +32,14 @@
 #endif
 
 /*
- * After above, since some of our port definitions affect what gets defined
- * tclInt.h and the Tcl port file.
+ * Internal interp flags compatibility - removed in Tcl 8.5 sources.
  */
-#include "tclInt.h"
+#ifndef ERR_IN_PROGRESS
+#define ERR_IN_PROGRESS	2
+#endif
+#ifndef ERROR_CODE_SET
+#define ERROR_CODE_SET 8
+#endif
 
 /*
  * Assert macro for use in TclX.  Some GCCs libraries are missing a function
@@ -558,7 +566,7 @@ int
 TclXOSGetSelectFnum _ANSI_ARGS_((Tcl_Interp *interp,
                                  Tcl_Channel channel,
                                  int         direction,
-                                 int        *fnumPtr));
+                                 unsigned int *fnumPtr));
 
 int
 TclXOSHaveFlock _ANSI_ARGS_((void));
