@@ -17,7 +17,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXwinOS.c,v 1.3 2002/09/26 00:23:30 hobbs Exp $
+ * $Id: tclXwinOS.c,v 1.7 2005/04/26 20:01:35 hobbs Exp $
  *-----------------------------------------------------------------------------
  * The code for reading directories is based on TclMatchFiles from the Tcl
  * distribution file win/tclWinFile.c
@@ -428,7 +428,7 @@ TclXOSsetitimer (Tcl_Interp *interp,
 void
 TclXOSsleep (unsigned seconds)
 {
-    Sleep(seconds*1000);
+    Tcl_Sleep(seconds*1000);
 }
 
 /*-----------------------------------------------------------------------------
@@ -513,8 +513,9 @@ TclXOSsystem (Tcl_Interp *interp,
 
 /*-----------------------------------------------------------------------------
  * TclX_OSlink --
- *   System dependent interface to link functionality, which is not available
- * on windows.
+ *
+ *   System dependent interface to link functionality, which is not
+ *   available on windows.
  *
  * Parameters:
  *   o interp - Errors returned in result.
@@ -585,8 +586,8 @@ TclXOSElapsedTime (clock_t *realTime,
 
 /*-----------------------------------------------------------------------------
  * TclXOSkill --
- *   System dependent interface to functionality, which is not available
- * on windows.
+ *   System dependent interface to terminate a process.  Apparently,
+ *   it's not possible to send a specific signal in windows?
  *
  * Parameters:
  *   o interp - Errors returned in result.
@@ -612,6 +613,7 @@ TclXOSkill (Tcl_Interp *interp,
     }
 
     TerminateProcess(processHandle, 7);
+    CloseHandle(processHandle);
     return TCL_OK;
 }
 
@@ -966,7 +968,7 @@ TclXOSftruncate (Tcl_Interp  *interp,
                               (char *) NULL);
 	return TCL_ERROR;
     }
-    pos = Tcl_Tell (channel);
+    pos = (int) Tcl_Tell (channel);
     if (SetFilePointer (handle, (LONG)newSize, NULL,
                         FILE_BEGIN) == 0xFFFFFFFF) {
         TclWinConvertError (GetLastError ());
@@ -1375,7 +1377,7 @@ int
 TclXOSGetSelectFnum (Tcl_Interp *interp,
                      Tcl_Channel channel,
                      int         direction,
-                     int        *fnumPtr)
+                     unsigned int *fnumPtr)
 {
     tclXwinFileType type;
     HANDLE handle = ChannelToHandle (channel, direction, &type);
@@ -1396,7 +1398,7 @@ TclXOSGetSelectFnum (Tcl_Interp *interp,
         return TCL_ERROR;
     }
 
-    *fnumPtr = (int) handle;
+    *fnumPtr = (unsigned int) handle;
     return TCL_OK;
     
 }
